@@ -54,9 +54,9 @@ func printable(s string) string {
 	s2 := ""
 	for _, r := range s {
 		if !unicode.IsPrint(r) {
-			if r < 0xFF {
+			if r <= 0xFF {
 				s2 += fmt.Sprintf("\\x%02X", r)
-			} else if r < 0xFFFF {
+			} else if r <= 0xFFFF {
 				s2 += fmt.Sprintf("\\u%04X", r)
 			} else {
 				s2 += fmt.Sprintf("\\U%08X", r)
@@ -138,7 +138,7 @@ func String(t *testing.T, got, wanted string, msgs ...interface{}) {
 
 func Float(t *testing.T, got, wanted float64, msgs ...interface{}) {
 	t.Helper()
-	if math.IsNaN(wanted) != math.IsNaN(got) || !math.IsNaN(wanted) && math.Abs(got-wanted) > Epsilon {
+	if math.IsNaN(wanted) != math.IsNaN(got) || !math.IsNaN(wanted) && Epsilon < math.Abs((got-wanted)/wanted) {
 		t.Fatalf("%s%s: %v != %v", trace(), message(msgs...), color(Red, got), color(Green, wanted))
 	}
 }
@@ -148,7 +148,7 @@ func Floats(t *testing.T, got, wanted []float64, msgs ...interface{}) {
 	equal := len(got) == len(wanted)
 	if equal {
 		for i := range got {
-			if math.IsNaN(wanted[i]) != math.IsNaN(got[i]) || !math.IsNaN(wanted[i]) && math.Abs(got[i]-wanted[i]) > Epsilon {
+			if math.IsNaN(wanted[i]) != math.IsNaN(got[i]) || !math.IsNaN(wanted[i]) && Epsilon < math.Abs((got[i]-wanted[i])/wanted[i]) {
 				equal = false
 				break
 			}
@@ -161,7 +161,7 @@ func Floats(t *testing.T, got, wanted []float64, msgs ...interface{}) {
 
 func FloatDiff(t *testing.T, got, wanted, diff float64, msgs ...interface{}) {
 	t.Helper()
-	if math.IsNaN(wanted) != math.IsNaN(got) || !math.IsNaN(wanted) && math.Abs(got-wanted) > diff {
+	if math.IsNaN(wanted) != math.IsNaN(got) || !math.IsNaN(wanted) && diff < math.Abs(got-wanted) {
 		t.Fatalf("%s%s: %v != %v", trace(), message(msgs...), color(Red, got), color(Green, fmt.Sprintf("%v ± %v", wanted, diff)))
 	}
 }
